use core::str;

use num::{pow::Pow, Integer};
use rand::{distributions::Standard, prelude::Distribution};

use crate::utils::*;

struct EllipticCurve<BigInt>
where
    BigInt: Integer,
{
    a: BigInt,
    b: BigInt,
    r#mod: BigInt,
}
impl<BigInt> EllipticCurve<BigInt>
where
    BigInt: Integer + std::marker::Copy + Pow<usize, Output = BigInt>,
    Standard: Distribution<BigInt>,
{
    pub fn new(a: BigInt, b: BigInt, r#mod: BigInt) -> Self {
        EllipticCurve { a, b, r#mod }
    }

    pub fn y(&self, x: BigInt) -> BigInt {
        (x.pow(3) + self.b * x + self.a) % self.r#mod
    }
    pub fn random(&self) -> (BigInt, BigInt) {
        let mut start = rand::random::<BigInt>() % self.r#mod;
        let i_count = rand::random::<i8>();

        for _ in 0..i_count {
            start = self.y(start);
        }

        (start, self.y(start))
    }
}

pub struct Person {
    pub name: String,
    private_key: Option<u32>,
    pub public_key: Option<u32>,
    pub shared_key: Option<u32>,
}
impl Person {
    pub fn new(name: &str) -> Self {
        Person {
            name: name.to_string(),
            private_key: None,
            public_key: None,
            shared_key: None,
        }
    }

    pub fn gen_keys(&mut self, start: u32, r#mod: u32) {
        let private = rand::random::<u32>();
        self.private_key = Some(private.into());
        let public = mod_pow(start, private.into(), r#mod);
        self.public_key = Some(public);
        eprintln!(
            "[{}] private: {} -> public: {}",
            self.name,
            self.private_key.unwrap(),
            self.public_key.unwrap()
        );
    }

    pub fn gen_shared(&mut self, public: u32, r#mod: u32) {
        self.shared_key = Some(mod_pow(public, self.private_key.unwrap(), r#mod))
    }

    pub fn diffie_hellman(p1: &mut Self, p2: &mut Self) {
        let m: u32 = 17;
        let curve = EllipticCurve::new(0, 7, m);
        let gen = curve.random().1;

        eprintln!(
            "Start params: x³+{}x+{}, mod {}, generator {}",
            0, 7, m, gen
        );

        p1.gen_keys(gen, m.into());
        p2.gen_keys(gen, m.into());
        p1.gen_shared(p2.public_key.unwrap(), m.into());
        p2.gen_shared(p1.public_key.unwrap(), m.into());
        assert_eq!(p1.shared_key, p2.shared_key);
        eprintln!(
            "[{}] Shared key for {}: {}",
            p1.name,
            p2.name,
            p1.shared_key.unwrap()
        );
    }

    fn xor_cipher(msg: &[u8], key: u32) -> Vec<u8> {
        let key_bytes = key.to_le_bytes();
        let key_len = key_bytes.len();

        msg.iter()
            .enumerate()
            .map(|(i, &byte)| byte ^ key_bytes[i % key_len])
            .collect()
    }

    pub fn send(&self, msg_raw: &[u8], target: &Self) {
        let encrypted_raw = Self::xor_cipher(msg_raw, self.shared_key.unwrap());
        let encrypted = str::from_utf8(&encrypted_raw).unwrap_or("not displayable");
        let msg = str::from_utf8(&msg_raw).unwrap_or("not displayable");
        eprintln!(
            "[{}] Sending message '{}' ({}) to {}",
            self.name, msg, encrypted, target.name
        );
        target.recv(&encrypted_raw, self);
    }
    fn recv(&self, msg_raw: &[u8], source: &Self) {
        let decrypted_raw = Self::xor_cipher(msg_raw, self.shared_key.unwrap());
        let decrypted = str::from_utf8(&decrypted_raw).unwrap_or("not displayable");
        let msg = str::from_utf8(&msg_raw).unwrap_or("not displayable");
        eprintln!(
            "[{}] Received message '{}' ({}) from {}",
            self.name, decrypted, msg, source.name
        );
    }
}
